/*
 * Copyright 2005-2006 Michelle Osmond Licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package message;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * This is the identifier for a message box. A box must have at least a name. If
 * the box is meant to be private to a particular portlet (like a PORTLET_SCOPE
 * portlet session attribute), the namespace should be the portlet id and
 * 'local' should be set to true, for the dynamic message wiring tools to work
 * properly.
 * 
 * @author Michelle Osmond
 * @version 1.2
 */
public class MessageBoxKey implements Comparable, java.io.Serializable
{
    private static final Log log = LogFactory.getLog(MessageBoxKey.class);

    protected String name = "";

    protected String namespace = "";

    protected boolean local = false;

    public MessageBoxKey(String name)
    {
        setName(name);
    }

    public MessageBoxKey(String name, String namespace)
    {
        this(name);
        setNamespace(namespace);
    }

    public MessageBoxKey(String name, String namespace, boolean local)
    {
        this(name, namespace);
        this.local = local;
    }

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        if (name == null)
        {
            name = "";
        }
        this.name = name;
    }

    public String getNamespace()
    {
        return namespace;
    }

    public void setNamespace(String namespace)
    {
        if (namespace == null)
        {
            namespace = "";
        }
        this.namespace = namespace;
    }

    protected void setLocalNamespace(String namespace)
    {
        this.namespace = namespace;
        this.local = true;
    }

    public boolean isLocal()
    {
        return this.local;
    }

    public boolean equals(Object obj)
    {
        return (obj != null && obj instanceof MessageBoxKey && this.name.equals(((MessageBoxKey) obj).getName()) && this.namespace
            .equals(((MessageBoxKey) obj).getNamespace()));
    }

    public int hashCode()
    {
        return this.toString().hashCode();
    }

    public String toString()
    {
        return "\"" + name + "\" (" + namespace + ")";
    }

    public String formString()
    {
        try
        {
            return "" + isLocal() + "[[" + URLEncoder.encode(getNamespace(), "UTF-8") + "]]"
                    + URLEncoder.encode(getName(), "UTF-8");
        }
        catch (UnsupportedEncodingException e)
        {
            log.error("Problem generating the encoded form string", e);
            return "";
        }
    }

    public static MessageBoxKey parseFormString(String s)
    {
        if (s == null || s.trim().length() == 0)
        {
            return null;
        }
        try
        {
            boolean isLocal = Boolean.valueOf(s.substring(0, s.indexOf("[["))).booleanValue();
            String namespace = s.substring(s.indexOf("[[") + 2, s.indexOf("]]"));
            String name = s.substring(s.indexOf("]]") + 2);

            return new MessageBoxKey(URLDecoder.decode(name, "UTF-8"), URLDecoder.decode(namespace, "UTF-8"), isLocal);

        }
        catch (Exception e)
        {
            log.error("MessageBoxKey: Unable to parse form string " + s, e);
            return null;
        }
    }

    public int compareTo(Object o)
    {
        if (o != null && o instanceof MessageBoxKey)
        {
            return this.toString().compareTo(((MessageBoxKey) o).toString());
        }
        return -1;
    }

}
