/*
 * Copyright 2005-2006 Michelle Osmond Licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package message;

import java.util.Map;
import java.util.Set;

/**
 * Storage and retrieval of portlet message mappings, and the messages. One
 * MessageCentre instance will be created per portlet webapp. The MessageStore
 * used (set in the constructor) will determine the messaging functionality: if
 * it uses a remote store such as a database, the messages will be available to
 * all portlet applications, but if it is a local store, they will only be
 * available to the current portlet application. Direct use of this class is
 * only necessary when you want to modify message mappings in code; to simply
 * load mappings from preferences, or send/receive messages, use
 * {@link MessageHelper}.
 * 
 * @author Michelle Osmond
 * @version 1.2
 */
public class MessageCentreImpl implements MessageCentre, java.io.Serializable
{

    protected MessageStore store = null;

    public void setStore(MessageStore store)
    {
        this.store = store;
    }

    /**
     * Set a portlet message key => message box mapping. This will overwrite any
     * existing input mapping.
     * 
     * @param portletID
     *            ID of the portlet which is receiving the input
     * @param name
     *            (local) name of the portlet input, referred to in the
     *            portlet's code
     * @param source
     *            the message box from which the portlet's input will read.
     */
    public void addInputMapping(String portletID, String name, MessageBoxKey source)
    {
        addInputMapping(makeLocalKey(portletID, name), source);
    }

    public void addInputMapping(MessageBoxKey key, MessageBoxKey source)
    {
        store.saveInputMapping(key, source);
    }

    /**
     * Remove a message key => message box mapping. As there can only be 1
     * mapping for each input, this will result in the default mapping: looking
     * for a local portlet message of the same name.
     */
    public void clearInputMapping(MessageBoxKey key)
    {
        addInputMapping(key, key);
    }

    /**
     * Remove all trace of the specified input message box. There will no longer
     * be even a default mapping.
     */
    public void deleteInput(MessageBoxKey key)
    {
        store.deleteInput(key);
    }

    /**
     * Add a portlet message key => message box mapping. This will be in
     * addition to any existing output mappings; the message will be copied to
     * all mapped output targets.
     * 
     * @param portletID
     *            ID of the portlet which is sending the output
     * @param name
     *            (local) name of the portlet output, referred to in the
     *            portlet's code
     * @param target
     *            the message box to which the portlet's output will copy its
     *            message.
     */
    public void addOutputMapping(String portletID, String name, MessageBoxKey target)
    {
        addOutputMapping(makeLocalKey(portletID, name), target);
    }

    public void addOutputMapping(MessageBoxKey key, MessageBoxKey target)
    {
        store.saveOutputMapping(key, target);
    }

    /**
     * Remove any existing output mappings for this key. The result will be that
     * the output still exists, but does not publish to anywhere.
     */
    public void clearAllOutputMappings(MessageBoxKey key)
    {
        store.clearAllOutputMappings(key);
    }

    /**
     * Remove all trace of the specified output message box. There will no
     * longer be even a default mapping, so it will no longer be possible to
     * publish messages to it.
     */
    public void deleteOutput(MessageBoxKey key)
    {
        store.deleteOutput(key);
    }

    /**
     * This will always return a Set which contains at least a key for a message
     * box corresponding to the provided portletID+name (it always outputs to
     * itself as well as any other registered mappings)
     * 
     * @return Set of MessageBoxKey's: the output keys registered for this
     *         portletID+name
     */
    public Set findOutputTargets(String portletID, String name)
    {
        return findOutputTargets(makeLocalKey(portletID, name));
    }

    public Set findOutputTargets(MessageBoxKey key)
    {
        return store.findOutputTargets(key);
    }

    /**
     * Get message for this portletID+name combination. If there is no mapping
     * to a message box defined for the specified portletID+name, it tries to
     * find a local message box of the same name. If there is no message,
     * returns null.
     */
    public Object get(String portletID, String name)
    {
        return get(findInputSource(portletID, name));
    }

    /**
     * Send message to this portletID+name combination. It will be delivered to
     * a local message box with this name. It will also be delivered to all
     * message boxes for which mappings have been registered for this
     * portletID+name using addMapping. It will replace any message already in
     * the boxes.
     */
    public void send(String portletID, String name, Object message)
    {
        send(findOutputTargets(portletID, name), message);
    }

    /**
     * Get the key of the source message box for this portletID+name
     * combination. If no mapping is found, returns a MessageBoxKey for this
     * portletID+name.
     */
    public MessageBoxKey findInputSource(String portletID, String name)
    {
        MessageBoxKey key = makeLocalKey(portletID, name);
        return findInputSource(key);
    }

    public MessageBoxKey findInputSource(MessageBoxKey key)
    {
        return store.getInputSource(key);
    }

    /**
     * Get message from this message box. If there is no message, returns null.
     */
    public Object get(MessageBoxKey boxKey)
    {
        return store.getMessage(boxKey);
    }

    /**
     * Send message to these message boxes. It will replace any message already
     * in the boxes, or remove the box if 'message' is null.
     * 
     * @param targetKeys
     *            Set of MessageBoxKeys to which the message should be sent
     */
    public void send(Set targetKeys, Object message)
    {
        store.sendMessage(targetKeys, message);
    }

    public Set getInputKeys(String portletID)
    {
        return store.getInputKeys(portletID);
    }

    public Set getOutputKeys(String portletID)
    {
        return store.getOutputKeys(portletID);
    }

    public Set getPublicBoxKeys()
    {
        return store.getPublicBoxKeys();
    }

    public Map getBoxes()
    {
        return store.getBoxes();
    }

    public Set getAllInputKeys()
    {
        return store.getAllInputKeys();
    }

    public Set getAllOutputKeys()
    {
        return store.getAllOutputKeys();
    }

    public boolean isValid()
    {
        return store.isValid();
    }

    // ---- own functions ----
    protected MessageBoxKey makeLocalKey(String portletID, String name)
    {
        return new MessageBoxKey(name, portletID, true);
    }
}
