/*
 * Copyright 2005-2006 Michelle Osmond Licensed under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law
 * or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */

package message.store;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import message.MessageBoxKey;
import message.MessageStore;

/**
 * Storage and retrieval of portlet message mappings, and the messages. The
 * MessageStore instance is kept within the MessageCentre instance for the
 * portlet webapp. This implementation just stores the messages locally in Maps,
 * and therefore if a BasicMessageStore is set as the store for a MessageCentre,
 * the messages will only be visible to portlets within the same webapp.
 * 
 * @author Michelle Osmond
 * @version 1.2
 */
public class BasicMessageStore implements MessageStore, java.io.Serializable
{

    protected Map inputKeyMap = null; // MessageBoxKey => MessageBoxKey

    protected Map outputKeyMap = null; // MessageBoxKey => Set of

    // MessageBoxKey's

    protected Map msgBoxes = null; // MessageBoxKey => Object (message value)

    public BasicMessageStore()
    {
    }

    // ---- Implement MessageStore methods. ------

    public void initialise(String sessionID)
    {
        inputKeyMap = new HashMap();
        outputKeyMap = new HashMap();
        msgBoxes = new HashMap();
    }

    public void saveInputMapping(MessageBoxKey key, MessageBoxKey source)
    {
        inputKeyMap.put(key, source);
    }

    public void deleteInput(MessageBoxKey key)
    {
        inputKeyMap.remove(key);
    }

    public void saveOutputMapping(MessageBoxKey key, MessageBoxKey target)
    {
        Set targetKeys = findOutputTargets(key);
        targetKeys.add(target);
    }

    public void clearAllOutputMappings(MessageBoxKey key)
    {
        outputKeyMap.put(key, new HashSet());
    }

    public void deleteOutput(MessageBoxKey key)
    {
        outputKeyMap.remove(key);
    }

    public Set findOutputTargets(MessageBoxKey key)
    {
        Set targetKeys = null;
        if (outputKeyMap.containsKey(key) && outputKeyMap.get(key) instanceof Set)
        {
            targetKeys = (Set) outputKeyMap.get(key);
        }
        else
        {
            targetKeys = new HashSet();
            outputKeyMap.put(key, targetKeys);
        }
        if (!targetKeys.contains(key))
        {
            targetKeys.add(key); // always send to 'self' (local message box)
        }
        return targetKeys;
    }

    public MessageBoxKey getInputSource(MessageBoxKey key)
    {
        MessageBoxKey boxKey = null;
        if (inputKeyMap.containsKey(key))
        {
            boxKey = (MessageBoxKey) inputKeyMap.get(key);
        }
        else
        {
            boxKey = key;
        }
        return boxKey;
    }

    public Object getMessage(MessageBoxKey key)
    {
        if (!msgBoxes.containsKey(key))
        {
            return null;
        }
        else
        {
            return msgBoxes.get(key);
        }
    }

    public void sendMessage(Set targetKeys, Object message)
    {
        if (targetKeys == null)
        {
            return;
        }
        for (Iterator it = targetKeys.iterator(); it.hasNext();)
        {
            MessageBoxKey targetKey = (MessageBoxKey) it.next();
            if (message != null)
            {
                msgBoxes.put(targetKey, message);
            }
            else
            {
                msgBoxes.remove(targetKey);
            }
        }
    }

    public Set getInputKeys(String portletID)
    {
        return getKeysWithNamespace(inputKeyMap.keySet(), portletID);
    }

    public Set getOutputKeys(String portletID)
    {
        return getKeysWithNamespace(outputKeyMap.keySet(), portletID);
    }

    public Set getPublicBoxKeys()
    {
        Set allBoxes = new TreeSet();
        // retrieve all possibly public boxes: look in output targets and input
        // sources
        for (Iterator it = outputKeyMap.values().iterator(); it.hasNext();)
        {
            Collection keys = (Collection) it.next();
            allBoxes.addAll(keys);
        }
        allBoxes.addAll(inputKeyMap.values());

        // look through the list
        Set keys = new TreeSet();
        for (Iterator it = allBoxes.iterator(); it.hasNext();)
        {
            Object key = it.next();
            if (key instanceof MessageBoxKey && !((MessageBoxKey) key).isLocal())
            {
                keys.add(key);
            }
        }
        return keys;
    }

    public Map getBoxes()
    {
        return msgBoxes;
    }

    public Set getAllInputKeys()
    {
        // HashMap's keyset is not serializable...
        return new TreeSet(inputKeyMap.keySet());
    }

    public Set getAllOutputKeys()
    {
        // HashMap's keyset is not serializable...
        return new TreeSet(outputKeyMap.keySet());
    }

    public boolean isValid()
    {
        return true;
    }

    // -------------------------------------------

    protected Set getKeysWithNamespace(Collection searchKeys, String ns)
    {
        if (ns == null)
        {
            ns = "";
        }
        Set keys = new TreeSet();
        for (Iterator it = searchKeys.iterator(); it.hasNext();)
        {
            Object key = it.next();
            if (key instanceof MessageBoxKey && ((MessageBoxKey) key).getNamespace().equals(ns))
            {
                keys.add(key);
            }
        }
        return keys;
    }
}
